-- JTO/blobstore: the sequence for the unique identifiers.
CREATE SEQUENCE of_jto_id
   START WITH 0
   INCREMENT BY 1
   MAXVALUE 9223372036854775807
   NOCYCLE
   CACHE 10
;

-- JTO/blobstore: table holding the JTO identifer/data.
CREATE TABLE OF_Jto
(
   JtoID VARCHAR(36) NOT NULL,
   Data BYTE NOT NULL,
   PRIMARY KEY (JtoID)
)
;

-- Event database: table holding the stored events.
CREATE TABLE OF_Events
(
   EventID INTEGER NOT NULL,
   ChannelID VARCHAR(36) NOT NULL,
   TStamp DATE NOT NULL,
   Data BYTE NOT NULL,
   FOREIGN KEY (ChannelID) REFERENCES OF_Jto (JtoID) ON DELETE CASCADE,
   PRIMARY KEY (EventID, ChannelID)
)
;

-- Event database: table holding the pushing/pushed records.
CREATE TABLE OF_Push
(
   ProxyID VARCHAR(36) NOT NULL,
   EventID INTEGER NOT NULL,
   ChannelID VARCHAR(36) NOT NULL,
   Status INTEGER NOT NULL,
   FOREIGN KEY (EventID, ChannelID) REFERENCES OF_Events (EventID, ChannelID) ON DELETE CASCADE,
   PRIMARY KEY (ProxyID, ChannelID, EventID)
)
;

-- JTO/blobstore: procedure to create a JTO record.
CREATE PROCEDURE of_create_jto (jid VARCHAR(36), d REFERENCES BYTE)
   INSERT INTO OF_Jto (JtoID, Data) VALUES (jid, d);
END PROCEDURE
;

-- JTO/blobstore: procedure to update a JTO record.
CREATE PROCEDURE of_update_jto (jid VARCHAR(36), d REFERENCES BYTE)
   UPDATE OF_Jto SET Data = d WHERE JtoID = jid;
END PROCEDURE
;

-- JTO/blobstore: procedure to delete a JTO record.
CREATE PROCEDURE of_delete_jto (jid VARCHAR(36))
   DELETE FROM OF_Jto WHERE JtoID = jid;
END PROCEDURE
;

-- JTO/blobstore: procedure to delete all JTO records.
CREATE PROCEDURE of_delete_all_jtos ()
   DELETE FROM OF_Jto;
END PROCEDURE
;

-- Event database: procedure to create an event record.
CREATE PROCEDURE of_create_event (eid INTEGER, cid VARCHAR(36), t DATE, d REFERENCES BYTE)
   INSERT INTO OF_Events (EventID, ChannelID, TStamp, Data) VALUES (eid, cid, t, d);
END PROCEDURE
;

-- Event database: procedure to delete an event record.
CREATE PROCEDURE of_delete_event (eid INTEGER, cid VARCHAR(36))
   DELETE FROM OF_Events WHERE EventID = eid AND ChannelID = cid;
END PROCEDURE
;

-- Event database: procedure to delete all event records for a channel.
CREATE PROCEDURE of_delete_events (cid VARCHAR(36))
   DELETE FROM OF_Events WHERE ChannelID = cid;
END PROCEDURE
;

-- Event database: procedure to delete all event records.
CREATE PROCEDURE of_delete_all_events ()
   DELETE FROM OF_Events;
END PROCEDURE
;

-- Event database: procedure to create a push record.
CREATE PROCEDURE of_create_push (pid VARCHAR(36), eid INTEGER, cid VARCHAR(36), s INTEGER)
   INSERT INTO OF_Push (ProxyID, EventID, ChannelID, Status) VALUES (pid, eid, cid, s);
END PROCEDURE
;

-- Event database: procedure to update a push record.
CREATE PROCEDURE of_update_push (pid VARCHAR(36), eid INTEGER, cid VARCHAR(36), s INTEGER)
   UPDATE OF_Push SET Status = s WHERE ProxyID = pid AND EventID = eid AND ChannelID = cid;
END PROCEDURE
;

-- Event database: procedure to delete a push record.
CREATE PROCEDURE of_delete_push (pid VARCHAR(36), eid INTEGER, cid VARCHAR(36))
   DELETE FROM OF_Push WHERE ProxyID = pid AND EventID = eid AND ChannelID = cid;
END PROCEDURE
;

-- Event database: procedure to delete all push records.
CREATE PROCEDURE of_delete_all_push ()
   DELETE FROM OF_Push;
END PROCEDURE
;
