-- JTO/blobstore: the sequence for the unique identifiers.
CREATE SEQUENCE of_jto_id
   INCREMENT BY 1
   MINVALUE 0
   MAXVALUE 9223372036854775807
   START WITH 0
   NOCYCLE
   CACHE 10
   ORDER
;

-- JTO/blobstore: table holding the JTO identifer/data.
CREATE TABLE OF_Jto
(
   JtoID VARCHAR2(36) NOT NULL,
   Data LONG RAW NOT NULL,
   PRIMARY KEY (JtoID)
)
;

-- Event database: table holding the stored events.
CREATE TABLE OF_Events
(
   EventID NUMBER NOT NULL,
   ChannelID VARCHAR2(36) NOT NULL,
   TStamp DATE NOT NULL,
   Data LONG RAW NOT NULL,
   FOREIGN KEY (ChannelID) REFERENCES OF_Jto (JtoID) ON DELETE CASCADE,
   PRIMARY KEY (EventID, ChannelID)
)
;

-- Event database: table holding the pushing/pushed records.
CREATE TABLE OF_Push
(
   ProxyID VARCHAR2(36) NOT NULL,
   EventID NUMBER NOT NULL,
   ChannelID VARCHAR2(36) NOT NULL,
   Status NUMBER NOT NULL,
   FOREIGN KEY (EventID, ChannelID) REFERENCES OF_Events (EventID, ChannelID) ON DELETE CASCADE,
   PRIMARY KEY (ProxyID, ChannelID, EventID)
)
;

-- JTO/blobstore: procedure to create a JTO record.
CREATE PROCEDURE of_create_jto (jid IN VARCHAR2, d IN LONG RAW)
IS
BEGIN
   INSERT INTO OF_Jto (JtoID, Data) VALUES (jid, d);
END of_create_jto;
/

-- JTO/blobstore: procedure to update a JTO record.
CREATE PROCEDURE of_update_jto (jid IN VARCHAR2, d IN LONG RAW)
IS
BEGIN
   UPDATE OF_Jto SET Data = d WHERE JtoID = jid;
END of_update_jto;
/

-- JTO/blobstore: procedure to delete a JTO record.
CREATE PROCEDURE of_delete_jto (jid IN VARCHAR2)
IS
BEGIN
   DELETE FROM OF_Jto WHERE JtoID = jid;
END of_delete_jto;
/

-- JTO/blobstore: procedure to delete all JTO records.
CREATE PROCEDURE of_delete_all_jtos
IS
BEGIN
   DELETE FROM OF_Jto;
END of_delete_all_jtos;
/

-- Event database: procedure to create an event record.
CREATE PROCEDURE of_create_event (eid IN NUMBER, cid IN VARCHAR2, t IN DATE, d IN LONG RAW)
IS
BEGIN
   INSERT INTO OF_Events (EventID, ChannelID, TStamp, Data) VALUES (eid, cid, t, d);
END of_create_event;
/

-- Event database: procedure to delete an event record.
CREATE PROCEDURE of_delete_event (eid IN NUMBER, cid IN VARCHAR2)
IS
BEGIN
   DELETE FROM OF_Events WHERE EventID = eid AND ChannelID = cid;
END of_delete_event;
/

-- Event database: procedure to delete all event records for a channel.
CREATE PROCEDURE of_delete_events (cid IN VARCHAR2)
IS
BEGIN
   DELETE FROM OF_Events WHERE ChannelID = cid;
END of_delete_events;
/

-- Event database: procedure to delete all event records.
CREATE PROCEDURE of_delete_all_events
IS
BEGIN
   DELETE FROM OF_Events;
END of_delete_all_events;
/

-- Event database: procedure to create a push record.
CREATE PROCEDURE of_create_push (pid IN VARCHAR2, eid IN NUMBER, cid IN VARCHAR2, s IN NUMBER)
IS
BEGIN
   INSERT INTO OF_Push (ProxyID, EventID, ChannelID, Status) VALUES (pid, eid, cid, s);
END of_create_push;
/

-- Event database: procedure to update a push record.
CREATE PROCEDURE of_update_push (pid IN VARCHAR2, eid IN NUMBER, cid IN VARCHAR2, s IN NUMBER)
IS
BEGIN
   UPDATE OF_Push SET Status = s WHERE ProxyID = pid AND EventID = eid AND ChannelID = cid;
END of_update_push;
/

-- Event database: procedure to delete a push record.
CREATE PROCEDURE of_delete_push (pid IN VARCHAR2, eid IN NUMBER, cid IN VARCHAR2)
IS
BEGIN
   DELETE FROM OF_Push WHERE ProxyID = pid AND EventID = eid AND ChannelID = cid;
END of_delete_push;
/

-- Event database: procedure to delete all push records.
CREATE PROCEDURE of_delete_all_push
IS
BEGIN
   DELETE FROM OF_Push;
END of_delete_all_push;
/
