-- Common Tables

CREATE TABLE OF_Servers
(
   SIDIndex INTEGER NOT NULL, /* server id index */
   SID CHAR(36) NOT NULL, /* server id uuid string */
   PRIMARY KEY (SID)
)
go

CREATE TABLE OF_Names
(
   Name TEXT NOT NULL, /* name string */
   NameHash INTEGER NOT NULL, /* name hash value */
   NameIndex INTEGER NOT NULL, /* name index */
   PRIMARY KEY (NameIndex)
)
go

CREATE TABLE OF_Objects
(
   OID CHAR(36) NOT NULL, /* object id uuid string */
   PIDIndex INTEGER NOT NULL, /* process id index */
   TStamp DATETIME NOT NULL, /* update timestamp */
   Data IMAGE NOT NULL, /* serialized object state */
   PRIMARY KEY (OID)
)
go

-- Procedure to find exisiting or create new process index

CREATE PROCEDURE of_get_server_index
   @id CHAR(36),
   @idx INTEGER output
AS
BEGIN
   SELECT @idx = (SELECT SIDIndex FROM OF_Servers WHERE SID = @id)
   IF @idx = NULL
   BEGIN
      SELECT @idx = (SELECT MAX (SIDIndex) + 1 FROM OF_Servers)
      IF @idx = NULL
      BEGIN
         SELECT @idx = 1
      END
      INSERT INTO OF_Servers (SIDIndex, SID) VALUES (@idx, @id)
   END
   RETURN 0
END
go

-- Procedure to delete an object

CREATE PROCEDURE of_delete_object
   @id CHAR(36)
AS
BEGIN
   DELETE FROM OF_Objects WHERE OID = @id
   RETURN 0
END
go

-- Procedure to delete objects

CREATE PROCEDURE of_delete_objects
   @pid INTEGER
AS
BEGIN
   DELETE FROM OF_Objects WHERE PIDIndex = @pid
   RETURN 0
END
go

-- Procedure to check whether an object exists

CREATE PROCEDURE of_object_exists
   @id CHAR(36),
   @ret INTEGER output
AS
BEGIN
   SELECT @ret = 0
   IF EXISTS (SELECT * FROM OF_Objects WHERE OID = @id)
   BEGIN
      SELECT @ret = 1
   END
   RETURN 0
END
go

-- Set procedures to chained mode
sp_procxmode of_get_server_index, chained
go
sp_procxmode of_delete_object, chained
go
sp_procxmode of_delete_objects, chained
go
sp_procxmode of_object_exists, chained
go