/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosClockService.examples;

import com.prismt.orb.*;
import org.omg.CosClockService.*;
import org.omg.CosPropertyService.*;

/**
 * Query a Mission clock.  This example assumes that
 * the ControlledClockSingleton is configured as a
 * Mission clock.
 */
public class MissionClockQuery
{
    private org.omg.CORBA.ORB orb = null;
    private ControlledClock controlledClock = null;

    /**
     * Public constructor
     */
    public MissionClockQuery (org.omg.CORBA.ORB orb)
    {
        this.orb = orb;
    }

    /**
     * Control a mission clock, querying the clock time
     */
    public void query ()
    {
       try
       {
           org.omg.CORBA.Object ref = orb.resolve_initial_references ("ControlledClock");
           controlledClock = ControlledClockHelper.narrow (ref);

           controlledClock.set (500000000000L);

           // display the time
           System.out.println ("Current time: " + controlledClock.current_time ());

           System.out.println ("Sleeping for two seconds while clock continues");

           Thread.sleep (2000);

           // pause the mission clock
           controlledClock.pause ();

           System.out.println ("Mission clock paused");

           // display the time
           System.out.println ("Current time: " + controlledClock.current_time ());

           System.out.println ("Mission clock still paused, sleeping for further 2 seconds");

           Thread.sleep (2000);

           // display the time
           System.out.println ("Current time: " + controlledClock.current_time ());

           // resume the mission clock
           controlledClock.resume ();

           System.out.println ("Mission clock resumed, sleeping for two seconds while clock continues");

           Thread.sleep (2000);

           // display the time
           System.out.println ("Current time: " + controlledClock.current_time ());
       }
       catch (org.omg.CORBA.ORBPackage.InvalidName invalidName)
       {
           System.out.println ("Failed to resolve ControlledClock");
           invalidName.printStackTrace ();
       }
       catch (TimeUnavailable unavailable)
       {
           System.out.println ("Time unavailable: ");
           unavailable.printStackTrace ();
       }
       catch (org.omg.CosClockService.ControlledClockPackage.NotSupported ns)
       {
           ns.printStackTrace ();
       }
       catch (Exception ex)
       {
           //catch all
           ex.printStackTrace ();
       }
    }

    /**
     * Program entry point
     */
    public static void main (String[] args)
    {
       org.omg.CORBA.ORB orb = ObjectAdapter.init (args);
       MissionClockQuery query = new MissionClockQuery (orb);

       query.query ();
    }
}
