/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosNaming.examples.LoadBalance;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;

import com.prismt.cos.CosNaming.NamingExtensions.LoadBalancerPackage.*;
import com.prismt.cos.CosNaming.NamingExtensions.*;
import com.prismt.orb.*;

/**
 * <code>LBServer</code> is a Server that creates the Server objects and binds
 * them to the NameService.
 *
 */
public class LBServer
{
   /**
    * Describe <code>main</code> method here.
    *
    * @param args[] a <code>String</code> value
    */
   public static void main (String args[])
   {
      System.out.print ("Initialising the orb.");

      //In order to make examples easier to run, for Orbacus-40 set a POAName
      if (com.prismt.openfusion.Version.getORB().
            toUpperCase().startsWith("ORBACUS-4"))
      {
         ObjectAdapter.setPOAName("OpenFusion.NamingLBServer");
      }

      org.omg.CORBA.ORB orb = ObjectAdapter.init (args);

      LBServer lbs = new LBServer (orb);
   }


   /**
    * Creates a new <code>LBServer</code> instance.
    *
    * @param orb_ an <code>org.omg.CORBA.ORB</code> value
    */
   public LBServer (org.omg.CORBA.ORB orb_)
   {
      org.omg.CORBA.ORB    orb         = null;
      org.omg.CORBA.Object nameRef     = null;
      org.omg.CORBA.Object LBRef       = null;
      NamingContext        rootContext = null;
      LoadBalancingFactory lbfactory   = null;
      LoadBalancer         lb          = null;
      ServerImpl           server1     = null;
      ServerImpl           server2     = null;
      ServerImpl           server3     = null;
      NameComponent        lbAlias[]   = null;
      NameComponent        lbObject[]  = null;

      orb = orb_;

      try
      {
         System.out.print ("Resolving name service.");
         nameRef = orb.resolve_initial_references ("NameService");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName in)
      {
         System.out.println ("Could not resolve Name Service");
         System.exit (1);
      }

      System.out.print ("Narrowing root context.");
      rootContext = NamingContextHelper.narrow (nameRef);

      try
      {
         System.out.print ("Resolving LoadBalancingFactory.");
         LBRef = orb.resolve_initial_references ("LoadBalancingFactory");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName in)
      {
         System.out.println ("Could not resolve Load Balancing Factory");
         System.exit (1);
      }

      System.out.print ("Narrowing LoadBalancingFactory.");
      lbfactory = LoadBalancingFactoryHelper.narrow (LBRef);


      try
      {
         System.out.print ("Creating load balancer with policy RoundRobin.");
         lb = lbfactory.createLoadBalancer ("RoundRobin");
      }
      catch (InvalidPolicy ip)
      {
         System.out.println ("Invalid Policy: Could not create loadbalancer");
         ip.printStackTrace ();
         System.exit (1);
      }
      catch (InvalidPlugin ip)
      {
         System.out.println ("Invalid Plugin");
         System.exit (1);
      }

      System.out.println ("Creating server objects.");
      server1 = new ServerImpl ();
      server2 = new ServerImpl ();
      server3 = new ServerImpl ();
      System.out.println ("Successfully created server objects.");

      System.out.println ("Adding server objects to load balancer.");
      lb.add (ServerHelper.narrow (ObjectAdapter.getObject (server1)));
      lb.add (ServerHelper.narrow (ObjectAdapter.getObject (server2)));
      lb.add (ServerHelper.narrow (ObjectAdapter.getObject (server3)));

      System.out.println ("Creating load balancing object and alias.");
      lbAlias = new NameComponent[1];
      lbAlias[0] = new NameComponent ();
      lbAlias[0].id = "LoadBalancingAlias";
      lbAlias[0].kind = "LoadBalancerExample";

      lbObject = new NameComponent[1];
      lbObject[0] = new NameComponent ();
      lbObject[0].id = "LoadBalancingObject";
      lbObject[0].kind = "LoadBalancerExample";

      try
      {
         try
         {
            System.out.println
               ("Binding load balancer into root context of NameService.");
            rootContext.bind (lbObject, lb);
         }
         catch (AlreadyBound ab)
         {
            System.out.println ("Failed - Already bound. Rebinding.");

            rootContext.rebind (lbObject, lb);
         }
      }
      // This try/catch block catches the exceptions from the bind and
      // rebind calls
      catch (NotFound e)
      {
         System.out.println ("Unexpected Exception: " + e);
         System.exit (1);
      }
      catch (InvalidName e)
      {
         System.out.println ("Unexpected Exception: " + e);
         System.exit (1);
      }
      catch (CannotProceed e)
      {
         System.out.println ("Unexpected Exception: " + e);
         System.exit (1);
      }
      System.out.println ("Success");


      try
      {
         try
         {
            System.out.println
               ("Binding load balancer alias into root context of NameService.");
            rootContext.bind (lbAlias, lb.getAlias ());
         }
         catch (AlreadyBound ab)
         {
            System.out.println ("Failed - Already bound. Rebinding.");
            rootContext.rebind (lbAlias, lb.getAlias ());
         }
      }
      // This try/catch block catches the exceptions from the bind and
      // rebind calls
      catch (NotFound e)
      {
         System.out.println ("Unexpected Exception: " + e);
         System.exit (1);
      }
      catch (InvalidName e)
      {
         System.out.println ("Unexpected Exception: " + e);
         System.exit (1);
      }
      catch (CannotProceed e)
      {
         System.out.println ("Unexpected Exception: " + e);
         System.exit (1);
      }
      System.out.println ("Success");


      System.out.println ("Entering Server Loop...");
      ObjectAdapter.ready ();
   }
}
