/******************************************************************************
 *                           - COPYRIGHT NOTICE -                             *
 *                                                                            *
 * The information contained herein and which follows is proprietary          *
 * information and is the property of Prism Technologies Limited. This        *
 * information is not to be divulged, released, copied, reproduced, or        *
 * conveyed to unauthorized personnel, companies, or other institutions       *
 * without the direct and expressed approval in writing of Prism Technologies *
 * Limited.                                                                   *
 *                                                                            *
 ******************************************************************************/

package com.prismt.cos.CosNaming.examples;

import org.omg.CORBA.*;
import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CosNaming.NamingContextExtPackage.*;
import com.prismt.cos.CosNaming.*;

/**
 * Naming Service Example
 */
public class NamingExample
{
   org.omg.CORBA.ORB orb;

   public NamingExample (org.omg.CORBA.ORB orb)
   {
      this.orb = orb;
   }

   public static void main (String[] args)
   {
      org.omg.CORBA.ORB orb = null;
      orb = com.prismt.orb.ORBAdapter.init (args);

      NamingContextExt rootContext = null;
      NamingContextExt root;

      NamingExample example = new NamingExample (orb);

      try
      {
         // Resolve NameService
         org.omg.CORBA.Object obj =
         orb.resolve_initial_references ("NameService");
         rootContext = NamingContextExtHelper.narrow (obj);
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println ("Unknown Service");
      }

      if (rootContext == null)
      {
         System.err.println ("Failed to resolve NameService");
         System.exit (1);
      }
      else
      {
         System.out.println ("Resolved NameService");
      }

      // Call class method to add and print objects from naming service
      root = example.bind_objects_to_service (rootContext);
      example.print_available_objects_from_service (root);

      System.exit (0);
   }

   /**
    * This method bind objects
    * to the naming service
    */
   public NamingContextExt bind_objects_to_service (NamingContextExt root)
   {
      final int elements = 10; // The 10 elements to be added

      NameComponent[] key = new NameComponent[1];
      StringBuffer x;

      for (int i = 0; i < elements; i++)
      {
         x = new StringBuffer ("Test id-%" + i);
         // Assign the values to the NameComponent
         key[0] = new NameComponent ();
         key[0].id = x.toString ();
         key[0].kind = "Test kind";

         System.out.println ("Binding: " + x.toString ()+" to NameService");

         try
         {
            // Bind the values to the naming service
            root.bind (key,root);
         }
         catch (AlreadyBound ex)
         {
            // if object AlreadyBound catches the exception
            System.out.println ("Caught AlreadyBound Exception");
         }
         catch (Exception ex)
         {
            System.err.println ("Error in bind. Caught: " + ex);
            System.exit (1);
         }
      }

      System.out.println ("Added : " + elements+" objects");
      return root;
   }

   /**
    * This method iterates through the Root of
    * the naming service and prints the first 5 objects to the screen
    */
   public void print_available_objects_from_service (NamingContextExt root)
   {
      // Setup variables for the IteratorHolder and ListHolder
      BindingIteratorHolder iter = new BindingIteratorHolder ();
      BindingListHolder list = new BindingListHolder ();

      try
      {
         // Query the naming service
         root.list (5, list, iter);

         // Iterate through and and print objects
         for (int i = 0; i < list.value.length; i++)
         {
            System.out.println ("List entry: " + i);
            System.out.println
               ("Name length: " + list.value[i].binding_name.length);
            System.out.println ("Name id: " + list.value[i].binding_name[0].id);
            System.out.println
               ("Name kind: " + list.value[i].binding_name[0].kind);
            System.out.println ("Bind type: " + list.value[i].binding_type);
         }
      }
      catch (Exception ex)
      {
         System.err.println ("Caught : " + ex);
      }
   }
}
