/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosNaming.examples.Video;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CosNaming.NamingContextExtPackage.*;

import com.prismt.orb.ORBAdapter;
import com.prismt.openfusion.examples.Video.*;

import java.util.Random;
import org.omg.CORBA.*;


/**
 * <code>VideoClient</code> represents a Client of the VideoServer
 * e.g. a TV Set.
 */
public class VideoClient
{
   /**
    * <code>main</code> method instantiates and runs the VideoClient.
    *
    * @param args a <code>String[]</code> value
    */
   public static void main (String [] args)
   {
      org.omg.CORBA.ORB orb = ORBAdapter.init (args);

      VideoClient theClient = new VideoClient (orb);
   }


   /**
    * <code>VideoClient</code> represents a Client that accesses the NameService
    * to retrieve objects bound in by the VideoServer.
    *
    * @param orb an <code>org.omg.CORBA.ORB</code> value
    */
   public VideoClient (org.omg.CORBA.ORB orb)
   {
      NamingContextExt     rootContext = null;
      org.omg.CORBA.Object object      = null;
      org.omg.CORBA.Object nameRef     = null;

      try
      {
         System.out.print ("Resolving name service.");
         nameRef = orb.resolve_initial_references ("NameService");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println ("Unknown Service");
      }

      System.out.print ("Narrowing root context.");
      rootContext = NamingContextExtHelper.narrow (nameRef);

      object = getObject (rootContext);

      getFilmDescription (object);

      System.out.println ("Completed Successfully");

      //Orbacus shutdown fix
      System.exit (0);
   }


   private org.omg.CORBA.Object getObject (NamingContextExt rootContext)
   {
      BindingIteratorHolder iter    = null;
      BindingListHolder     list    = null;
      org.omg.CORBA.Object  listObj = null;
      org.omg.CORBA.Object  object  = null;
      NamingContext         nc      = null;

      int    randNo   = (new Random ()).nextInt (3);
      String films[]  = new String[3];

      films[0]= "Videos/Films/Pulp Fiction";
      films[1]= "Videos/Films/ET";
      films[2]= "Videos/Films/The Shining";


      // List bindings in a NamingContext
      iter = new BindingIteratorHolder ();
      list = new BindingListHolder ();

      rootContext.list (10, list, iter);

      try
      {
         // Iterate through and and print objects
         for (int i = 0; i < list.value.length; i++)
         {
            System.out.println ("List entry: " + i);
            System.out.println ("Name id: " + list.value[i].binding_name[0].id);
            System.out.println
               ("Name kind: " + list.value[i].binding_name[0].kind);
            System.out.println
               ("Bind type: " + list.value[i].binding_type.value ());

            System.out.println ("Attempt to resolve the found name");

            listObj = rootContext.resolve (list.value[i].binding_name);

            System.out.println ("Found: " + listObj);
         }

         // Note resolve_str is being used instead of resolve
         object = rootContext.resolve_str (films[randNo]);
      }
      catch (NotFound e)
      {
         System.out.println ("Resolve Failed: " + e);
         e.printStackTrace ();
         System.exit (1);
      }
      catch (CannotProceed e)
      {
         System.out.println ("Resolve Failed: " + e);
         e.printStackTrace ();
         System.exit (1);
      }
      catch (InvalidName e)
      {
         System.out.println ("Resolve Failed: " + e);
         e.printStackTrace ();
         System.exit (1);
      }
      return object;
   }


   private void getFilmDescription (org.omg.CORBA.Object obj)
   {
      // Narrow down to required Film object
      Film film = FilmHelper.narrow (obj);

      // Call methods on Film stub
      String filmDesc = film.description ();
      String filmCert = film.certificate ();

      System.out.println
         ("The film is " + filmDesc + " and it has a certificate " + filmCert);
   }
}
