/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosNaming.examples.Video;


import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;

import com.prismt.orb.*;
import com.prismt.openfusion.examples.Video.*;

import org.omg.CORBA.*;


/**
 * <code>VideoServer</code> is a VideoMachine that creates CORBA
 * objects representing films and binds them into the NameService.
 *
 */
public class VideoServer
{
   /**
    * <code>main</code> instantiates and runs the VideoServer
    *
    * @param args a <code>String[]</code> value
    */
   public static void main (String [] args)
   {
      System.out.println ("Initialising the orb.");

      //In order to make examples easier to run, for Orbacus-40 set a POAName
      if (com.prismt.openfusion.Version.getORB().
            toUpperCase().startsWith("ORBACUS-4"))
      {
         ObjectAdapter.setPOAName("OpenFusion.NamingVideoServer");
      }

      org.omg.CORBA.ORB orb = ObjectAdapter.init (args);

      VideoServer videoServer = new VideoServer (orb);


      System.out.println ("Entering Server Loop...");

      // non blocking mode so that user can hit Enter to exit program
      //
      ObjectAdapter.ready (false);

      try
      {
         System.out.println( "Press Enter to exit program...");
         int nextByte = System.in.read();
      }
      catch (java.io.IOException ioEx)
      {
      }

      // shutdown ObjectAdapter and deactivate all active objects
      //
      ObjectAdapter.shutdown ();
      System.exit (0);
   }


   /**
    * <code>VideoServer</code> represents a VideoMachine that creates CORBA
    * objects representing films and binds them into the NameService.
    *
    * @param orb an <code>ORB</code> value
    */
   public VideoServer (ORB orb)
   {
      NamingContext        rootContext = null;
      org.omg.CORBA.Object nameRef     = null;

      try
      {
         System.out.println ("Resolving name service.");
         nameRef = orb.resolve_initial_references ("NameService");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println ("Unknown Service");
         System.exit (0);
      }

      System.out.println ("Narrowing root context.");
      rootContext = NamingContextHelper.narrow (nameRef);

      System.out.println ("Adding NamingContexts and objects to Name Service");

      createHierachy (rootContext);
   }


   private Film getFilm (String title, String certificate)
   {
      org.omg.CORBA.Object corbaObject = null;
      FilmImpl             film        = null;
      Film                 filmObj     = null;

      film = new FilmImpl (title, certificate);
      corbaObject = ObjectAdapter.createTransient (film);
      filmObj = FilmHelper.narrow (corbaObject);

      return filmObj;
   }


   private void createHierachy (NamingContext rootContext)
   {
      NamingContext filmContext  = null;
      NamingContext videoContext = null;
      NameComponent videoName[]  = null;
      NameComponent filmName[]   = null;
      NameComponent name[]       = null;
      Film          film         = null;

      videoName = new NameComponent[1];
      videoName[0] = new NameComponent ("Videos", "");

      filmName = new NameComponent[1];
      filmName[0] = new NameComponent ("Films","");

      name = new NameComponent[1];

      try
      {
         try
         {
            System.out.println ("Creating context for videos.");
            videoContext = rootContext.new_context ();

            System.out.println ("Binding into NameService.");
            rootContext.bind_context (videoName, videoContext);
         }
         catch (AlreadyBound e)
         {
            System.out.println ("Failed - Already bound. Rebinding.");
            rootContext.rebind_context (videoName, videoContext);
         }

         try
         {
            System.out.println ("Binding context for films.");
            filmContext = videoContext.new_context ();

            videoContext.bind_context (filmName, filmContext);
         }
         catch (AlreadyBound e)
         {
            System.out.println ("Failed - Already bound. Rebinding.");
            videoContext.rebind_context (filmName, filmContext);
         }


         System.out.println ("Binding films into NameService.");


         film = getFilm ("ET - kids spaceman", "pg");
         name[0] = new NameComponent ("ET", "");
         try
         {
            filmContext.bind (name, film);
         }
         catch (AlreadyBound e)
         {
            System.out.println
               ("Failed to bind film - Already bound. Rebinding.");
            filmContext.rebind (name, film);
         }

         film = getFilm ("Pulp Fiction - gangster comic capers", "18");
         name[0] = new NameComponent ("Pulp Fiction", "");
         try
         {
            filmContext.bind (name, film);
         }
         catch (AlreadyBound e)
         {
            System.out.println
               ("Failed to bind film - Already bound. Rebinding.");
            filmContext.rebind (name, film);
         }


         film = getFilm ("The Shining - scary movie", "18");
         name[0] = new NameComponent ("The Shining", "");
         try
         {
            filmContext.bind (name, film);
         }
         catch (AlreadyBound e)
         {
            System.out.println
               ("Failed to bind film - Already bound. Rebinding.");
            filmContext.rebind (name, film);
         }

         System.out.println ("Completed Binding.");
      }
      // This try/catch block catches the exceptions from the bind and
      // rebind calls
      catch (NotFound e)
      {
         System.out.println ("Unexpected Exception: " + e);
         System.exit (1);
      }
      catch (InvalidName e)
      {
         System.out.println ("Unexpected Exception: " + e);
         System.exit (1);
      }
      catch (CannotProceed e)
      {
         System.out.println ("Unexpected Exception: " + e);
         System.exit (1);
      }
   }
}
