/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosNotification.examples.Benchmark;

import java.io.*;

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;


/**
 * The benchmark sequence push supplier.
 */
public class BenchmarkSequenceSupplier extends BenchmarkSupplier
                                       implements SequencePushSupplierOperations,
                                                  java.io.Serializable
{
   private SequenceProxyPushConsumer proxyConsumer = null;
   private SequencePushSupplier supplier = null;


   /**
    * Main method.  Creates an instance of the supplier and sends the events.
    * @param args  The command line arguments.
    */
   public static void main (String[] args)
   {
      BenchmarkSequenceSupplier supp = new BenchmarkSequenceSupplier ();
      supp.initialise (args);
      supp.connect ();

      System.out.println (supp + ": ready: press return to send events");
      BufferedReader br =
         new BufferedReader (new InputStreamReader (System.in));
      try
      {
         br.readLine ();
      }
      catch(IOException ioe)
      {
      }

      try
      {
         System.out.println (supp + ": now sending events...");
         supp.sendEvents ();
      }
      catch (Exception ex)
      {
         System.out.println ("Exception thrown when sending events");
         ex.printStackTrace ();
         supp.disconnect ();
         System.exit (1);
      }
      
      // If reporting, always display the final rate.
      if (supp.reportThreshold != 0)
      {
         if (supp.lastReportEventCount != supp.eventsSent)
         {
            supp.report (supp.eventsSent);
         }
      }

      supp.disconnect ();
      System.exit (0);
   }


   /**
    * @param args  The command line arguments.
    */
   public BenchmarkSequenceSupplier ()
   {
      super ();
      isSupplier = true;
      clientType = ClientType.SEQUENCE_EVENT;
      setID ("BenchmarkSequenceSupplier");
   }


   public void setProxyReference (org.omg.CORBA.Object proxy)
   {
      proxyConsumer = SequenceProxyPushConsumerHelper.narrow (proxy);
      m_proxyReferenceSet = true;
   }


   public void connect ()
   {
      try
      {
         ProxyConsumer pc = null;
         org.omg.CORBA.IntHolder idHolder = new org.omg.CORBA.IntHolder ();
         pc = suppAdmin.obtain_notification_push_consumer (clientType, idHolder);
         proxyConsumer = SequenceProxyPushConsumerHelper.narrow (pc);
         applyQoSFilters (proxyConsumer);
         
         if (supplier == null)
         {
            supplier = SequencePushSupplierHelper.narrow (thisRef);
         }
         proxyConsumer.connect_sequence_push_supplier (supplier);
      }
      catch (Exception ex)
      {
         System.err.println (this + ": failed to connect -- " + ex);
         ex.printStackTrace ();
         System.exit (-1);
      }
   }


   /**
    * Disconnects this client from the event channel.
    */
   public void disconnect ()
   {
      if (proxyConsumer != null)
      {
         try
         {
            proxyConsumer.disconnect_sequence_push_consumer ();
         }
         catch (Exception ex)
         {
            System.err.println (this + ": failed to disconnect -- " + ex);
            ex.printStackTrace ();
         }
      }
   }


   public void sendEvents ()
         throws Exception
   {
      StructuredEvent[] events = new StructuredEvent[batchSize];

      startTime = System.currentTimeMillis ();

      int eventID = 1;
      eventsSent = 0;
      
      while (true)
      {
         // Stop if the specified number of events have been sent.
         if (eventLimit != 0)
         {
            if (eventsSent >= eventLimit)
            {
               System.out.println (this + ": sent required number of events -- " + eventsSent);
               break;
            }
         }

         for (int i = 0 ; i < batchSize ; i++)
         {
            events[i] = eventFactory.getStructuredEvent (persistenceSpecified, persistence, orb, eventID);
            eventID++;
         }
         
         proxyConsumer.push_structured_events (events);

         currentTime = System.currentTimeMillis ();

         reportIfNeeded (eventsSent, events.length);
         delayIfNeeded (eventsSent, events.length);
         eventsSent += events.length;
      }
   }
}


// EOF
