/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosNotification.examples.Benchmark;

import java.io.*;

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;
import org.omg.CosNotifyFilter.*;

import com.prismt.cos.CosNotification.NotificationExtensions.AutoSequenceBatchSize;
import com.prismt.cos.CosNotification.NotificationExtensions.AutoSequenceTimeout;


/**
 * The benchmark structured push consumer.
 */
public class BenchmarkStructuredConsumer extends BenchmarkConsumer
                                         implements StructuredPushConsumerOperations,
                                                    java.io.Serializable
{
   private StructuredProxyPushSupplier proxySupplier = null;
   private StructuredPushConsumer consumer = null;

   /** Holds the ID of the last event received (-1 for no previous events). */
   private int lastID = -1;


   /**
    * Main method.  Creates an instance of the consumer and waits for events.
    * @param args  The command line arguments.
    */
   public static void main (String[] args)
   {
      BenchmarkStructuredConsumer cons = new BenchmarkStructuredConsumer ();
      cons.initialise (args);
      cons.connect ();

      System.out.println (cons + ": ready: press return to exit");
      BufferedReader br =
         new BufferedReader (new InputStreamReader (System.in));
      try
      {
         br.readLine ();
      }
      catch (IOException ioe)
      {
      }

      // If reporting, always display the final rate.
      if (cons.reportThreshold != 0)
      {
         if (cons.lastReportEventCount != cons.eventsReceived)
         {
            cons.report (cons.eventsReceived);
         }
      }
      
      cons.disconnect ();
      System.exit (0);
   }


   public BenchmarkStructuredConsumer ()
   {
      super ();
      isSupplier = false;
      clientType = ClientType.STRUCTURED_EVENT;
      setID ("BenchmarkStructuredConsumer");
   }


   public void setProxyReference (org.omg.CORBA.Object proxy)
   {
      proxySupplier = StructuredProxyPushSupplierHelper.narrow (proxy);
      m_proxyReferenceSet = true;
   }


   protected void prepareQoSProperties ()
   {
      Property[] qos = new Property[2];
      qos[0] = new Property ();
      qos[0].name = AutoSequenceBatchSize.value;
      qos[0].value = orb.create_any ();
      qos[0].value.insert_long (autoSequenceBatchSize);
      qos[1] = new Property ();
      qos[1].name = AutoSequenceTimeout.value;
      qos[1].value = orb.create_any ();
      qos[1].value.insert_ulonglong ((long) autoSequenceTimeout);
      setQoSProperties (qos);
   }


   public void connect ()
   {
      try
      {
         ProxySupplier ps = null;
         org.omg.CORBA.IntHolder idHolder = new org.omg.CORBA.IntHolder ();
         ps = consAdmin.obtain_notification_push_supplier (clientType, idHolder);
         proxySupplier = StructuredProxyPushSupplierHelper.narrow (ps);
         applyQoSFilters (proxySupplier);
         
         if (consumer == null)
         {
            consumer = StructuredPushConsumerHelper.narrow (thisRef);
         }
         proxySupplier.connect_structured_push_consumer (consumer);
      }
      catch (Exception ex)
      {
         System.err.println (this + ": failed to connect -- " + ex);
         ex.printStackTrace ();
         System.exit (-1);
      }
   }


   /**
    * Disconnects this client from the event channel.
    */
   public void disconnect ()
   {
      if (proxySupplier != null)
      {
         try
         {
            proxySupplier.disconnect_structured_push_supplier ();
         }
         catch (Exception ex)
         {
            System.err.println (this + ": failed to disconnect -- " + ex);
            ex.printStackTrace ();
         }
      }
   }


   /**
    * Method called by the proxy supplier to send an event.
    * @param event  The structured event.
    */
   public void push_structured_event (StructuredEvent event)
   {
      eventsReceived++;

      // Perform event ordering check in a try/catch, in case an event without
      // the expected body arrives.
      try
      {
         int eventID = event.remainder_of_body.extract_long ();

         if (checkOrdering)
         {
            if (lastID != -1)
            {
               if (eventID != (lastID+1))
               {
                  System.err.println (this + ": unexpected event: event ID = " +
                                      eventID + ", but last ID = " + lastID);
               }
            }
            lastID = eventID;
         }
      }
      catch (Throwable th)
      {
         // Ignore.
      }

      if (startTime == -1)
      {
         startTime = System.currentTimeMillis ();
      }

      currentTime = System.currentTimeMillis ();

      if (displayReceivedEvents)
      {
         displayStructuredEvent (event);
      }

      reportIfNeeded (eventsReceived);
      delayIfNeeded (eventsReceived);
   }
}


// EOF
