/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosNotification.examples.Benchmark;

import java.io.*;

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;

import com.prismt.cos.CosNotification.NotificationExtensions.AutoSequenceBatchSize;
import com.prismt.cos.CosNotification.NotificationExtensions.AutoSequenceTimeout;


/**
 * The benchmark structured push supplier.
 */
public class BenchmarkStructuredSupplier extends BenchmarkSupplier
                                         implements StructuredPushSupplierOperations,
                                                    java.io.Serializable
{
   private StructuredProxyPushConsumer proxyConsumer = null;
   private StructuredPushSupplier supplier = null;


   /**
    * Main method.  Creates an instance of the supplier and sends the events.
    * @param args  The command line arguments.
    */
   public static void main (String[] args)
   {
      BenchmarkStructuredSupplier supp = new BenchmarkStructuredSupplier ();
      supp.initialise (args);
      supp.connect ();

      System.out.println (supp + ": ready: press return to send events");
      BufferedReader br =
         new BufferedReader (new InputStreamReader (System.in));
      try
      {
         br.readLine ();
      }
      catch(IOException ioe)
      {
      }

      try
      {
         System.out.println (supp + ": now sending events...");
         supp.sendEvents ();
      }
      catch (Exception ex)
      {
         System.out.println ("Exception thrown when sending events");
         ex.printStackTrace ();
         supp.disconnect ();
         System.exit (1);
      }
      
      // If reporting, always display the final rate.
      if (supp.reportThreshold != 0)
      {
         if (supp.lastReportEventCount != supp.eventsSent)
         {
            supp.report (supp.eventsSent);
         }
      }

      supp.disconnect ();
      System.exit (0);
   }


   /**
    * @param args  The command line arguments.
    */
   public BenchmarkStructuredSupplier ()
   {
      super ();
      isSupplier = true;
      clientType = ClientType.STRUCTURED_EVENT;
      setID ("BenchmarkStructuredSupplier");
   }


   public void setProxyReference (org.omg.CORBA.Object proxy)
   {
      proxyConsumer = StructuredProxyPushConsumerHelper.narrow (proxy);
      m_proxyReferenceSet = true;
   }


   protected void prepareQoSProperties ()
   {
      Property[] qos = new Property[2];
      qos[0] = new Property ();
      qos[0].name = AutoSequenceBatchSize.value;
      qos[0].value = orb.create_any ();
      qos[0].value.insert_long (autoSequenceBatchSize);
      qos[1] = new Property ();
      qos[1].name = AutoSequenceTimeout.value;
      qos[1].value = orb.create_any ();
      qos[1].value.insert_ulonglong ((long) autoSequenceTimeout);
      setQoSProperties (qos);
   }


   public void connect ()
   {
      try
      {
         ProxyConsumer pc = null;
         org.omg.CORBA.IntHolder idHolder = new org.omg.CORBA.IntHolder ();
         pc = suppAdmin.obtain_notification_push_consumer (clientType, idHolder);
         proxyConsumer = StructuredProxyPushConsumerHelper.narrow (pc);
         applyQoSFilters (proxyConsumer);

         if (supplier == null)
         {
            supplier = StructuredPushSupplierHelper.narrow (thisRef);
         }
         proxyConsumer.connect_structured_push_supplier (supplier);
      }
      catch (Exception ex)
      {
         System.err.println (this + ": failed to connect -- " + ex);
         ex.printStackTrace ();
         System.exit (-1);
      }
   }


   /**
    * Disconnects this client from the event channel.
    */
   public void disconnect ()
   {
      if (proxyConsumer != null)
      {
         try
         {
            proxyConsumer.disconnect_structured_push_consumer ();
         }
         catch (Exception ex)
         {
            System.err.println (this + ": failed to disconnect -- " + ex);
            ex.printStackTrace ();
         }
      }
   }
  

   public void sendEvents ()
         throws Exception
   {
      StructuredEvent event = null;

      startTime = System.currentTimeMillis ();

      int eventID = 1;
      eventsSent = 0;
      boolean eventRejected = false;
      
      while (true)
      {
         // Stop if the specified number of events have been sent.
         if (eventLimit != 0)
         {
            if (eventsSent == eventLimit)
            {
               System.out.println (this + ": sent required number of events -- " + eventsSent);
               break;
            }
         }
         
         event = eventFactory.getStructuredEvent (persistenceSpecified, persistence, orb, eventID);
         
         try
         {
            proxyConsumer.push_structured_event (event);
         }
         catch (org.omg.CORBA.IMP_LIMIT il)
         {
            eventRejected = true;
            numRejectedEvents++;
            sleep (-1);
            eventsSent--;//decrement to nullify increment
         }

         currentTime = System.currentTimeMillis ();
         eventID++;
         eventsSent++;

         reportIfNeeded (eventsSent);
         
         if (eventRejected == false)
         {
            delayIfNeeded (eventsSent);
         }
      }
   }
}


// EOF
