/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 2.5
 */

package com.prismt.cos.CosNotification.examples.Filter;

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyFilter.*;


/**
 * Example Filter: <code>MyFilter</code> shows a simple implemantion of a
 * custom filter. It checks the height property of a structured event is
 * in the interval between 10 and 20 and the length property of a strucutured
 * event is in the interval between 20 and 40.
 */
public class MyFilter implements FilterOperations
{
   /**
    * Creates a new <code>MyFilter</code> instance.
    *
    * @param grammer a <code>String</code> value
    * @exception InvalidGrammar if an error occurs
    */
   public MyFilter (String grammer) throws InvalidGrammar
   {
   }

   /**
    * The <code>add_constraints</code> method is invoked by a client in
    * order associate one or more new constraints with the filter.
    * The method is not implemented in this case.
    *
    * @param constraint_list a <code>ConstraintExp[]</code>
    * sequence of constraints
    * @return a <code>ConstraintInfo[]</code> a sequence of constraints
    * and their unique identifiers.
    * @exception InvalidConstraint if the constaint is not valid
    */
   public synchronized ConstraintInfo[]
   add_constraints(ConstraintExp[] constraint_list)  throws InvalidConstraint
   {
      throw new org.omg.CORBA.NO_IMPLEMENT ();
   }

   /**
    * The  <code>modify_constraints</code> method is invoked by a client to
    * modifiy the constraints associated with the filter. The method is not
    * implemented in this case.
    *
    * @param del_list an <code>int[]</code> a sequence of unique identifiers
    * associated with the constraints. These are removed from the filter.
    * @param modify_list a <code>ConstraintInfo[]</code> a sequnce of unique
    * indentifiers and the constraints associated with them. These replace the
    * constriants that are already available in the filter.
    * @exception InvalidConstraint if the constriant is not valid.
    * @exception ConstraintNotFound if the constraint is not within the filter.
    */
   public synchronized void modify_constraints
      (int[] del_list, ConstraintInfo[] modify_list)
      throws InvalidConstraint, ConstraintNotFound
   {
      throw new org.omg.CORBA.NO_IMPLEMENT ();
   }

   /**
    * The <code>get_constraints</code> method will return a required subset of
    * the constraint implemented by the filter. This method is not implemented
    * in this case.
    * @param id_list an <code>int[]</code> sequence of unique identifiers
    * corresponding to the subset of constraints required.
    * @return a <code>ConstraintInfo[]</code> the set of constraints associated
    * with the unique identifiers.
    * @exception ConstraintNotFound if the input value does not match the
    * constraints available in the filer.
    */
   public synchronized ConstraintInfo[] get_constraints
      (int[] id_list)
      throws ConstraintNotFound
   {
      throw new org.omg.CORBA.NO_IMPLEMENT ();
   }

   /**
    * The  <code>get_all_constraints</code> method returns all the constraints
    * contained in the filter. This method is not implemented in this case.
    *
    * @return a <code>ConstraintInfo[]</code> the constraints in the filter.
    */
   public synchronized ConstraintInfo[] get_all_constraints ()
   {
      throw new org.omg.CORBA.NO_IMPLEMENT ();
   }

   /**
    * The <code>remove_all_constraints</code> method removes all the
    * constraints associated with the filter. This method is not implemented
    * in this case.
    */
   public synchronized void remove_all_constraints ()
   {
      throw new org.omg.CORBA.NO_IMPLEMENT ();
   }

   /**
    * The <code>destroy</code> method destroys the target filter object.
    * This method is not implemented in this case.
    *
    */
   public synchronized void destroy ()
   {
   }

   /**
    * The <code>match</code> method evaluates the filter constraints against
    * an event supplied to the channel in the form of an org.omg.CORBA.Any.
    * This method is not implemented in this case.
    *
    * @param event an <code>org.omg.CORBA.Any</code> the event supplied to the
    * channel
    * @return a <code>boolean</code> value which is true when the event
    * satisfies a filter constraint and false if it doesn't/
    * @exception UnsupportedFilterableData if the input parameter contains
    * data the match operation can not handle.
    */
   public synchronized boolean match (org.omg.CORBA.Any event)
   throws UnsupportedFilterableData
   {
      throw new org.omg.CORBA.NO_IMPLEMENT ();
   }

   /**
    * The <code>match_structured</code> method evaluates the filter
    * constraints against a structured event instance supplied to the channel.
    *
    * @param event a <code>StructuredEvent</code> the structured event to
    * be evaluated.
    * @return a <code>boolean</code> value, true if the event satisfies the
    * filter and false if it doesn't/
    * @exception UnsupportedFilterableData if the input parameter contains data
    * this method is not designed to handle.
    */
   public synchronized boolean match_structured (StructuredEvent event)
   throws UnsupportedFilterableData
   {
      /* The filterable data part of the event */
      Property[] props = event.filterable_data;
      /* booleans to check the height and length properties of the event*/
      boolean lengthOK = false;
      boolean heightOK = false;

      for (int i = 0; i < props.length; i++)
      {
         if (props[i].name.equals ("length"))
         {
            /* Get the value of the length property and check it
             * is between 20 and 40
             */
            int length = props[i].value.extract_long ();
            if (length > 20 && length < 40)
            {
               lengthOK = true;
            }
         }
         else if (props[i].name.equals ("height"))
         {
            /* Get the value of the height property and check it is
             * between 10 and 20
             */
            int height = props[i].value.extract_long ();
            if (height > 10 && height < 20)
            {
               heightOK = true;
            }
         }
      }
       return (lengthOK && heightOK);
   }

   /**
    * The <code>match_typed</code> method is used to evaluate the filter
    * constraints against a typed event. This method is not implemented in this
    * case.
    *
    * @param event a <code>Property[]</code> name value pairs containing the
    * contents of the event to be evaluated.
    * @return a <code>boolean</code> value, true if the event satisfies the
    * filter constraint and false if it doesn't.
    * @exception UnsupportedFilterableData if the input parameter contains
    * data the match operation can not handle.
    */
   public synchronized boolean match_typed (Property[] event)
   throws UnsupportedFilterableData
   {
      throw new org.omg.CORBA.NO_IMPLEMENT ();
   }

   /**
    * The <code>attach_callback</code> method cause the filter to invoke
    * subscription change when the add_constrains or modify_constraints
    * method is called. This method has not been implemented in this case.
    *
    * @param callback a <code>NotifySubscribe</code> reference to the callback
    * object
    * @return an <code>int</code> vaue assigned to this callback
    */
   public synchronized int attach_callback (NotifySubscribe callback)
   {
      throw new org.omg.CORBA.NO_IMPLEMENT ();
   }

   /**
    * The <code>detach_callback</code> method removes an object from the
    * list of objects that should receive callbacks. This method is not
    * implemented in this case..
    *
    * @param callback an <code>int</code> value associated with a callback
    * object
    * @exception CallbackNotFound if the number does not correspond to an
    * attached object.
    */
   public synchronized void detach_callback (int callback)
   throws CallbackNotFound
   {
      throw new org.omg.CORBA.NO_IMPLEMENT ();
   }

   /**
    * The <code>get_callbacks</code> method returns all the unique identifiers
    * associated with the callback objects. This method is not implemented
    * in this case.
    *
    * @return an <code>int[]</code> value
    */
   public synchronized int [] get_callbacks ()
   {
      throw new org.omg.CORBA.NO_IMPLEMENT  ();
   }

   /**
    * The <code>constraint_grammar</code> method sets an attribute to
    * identify the particular grammar used by the filter object.
    *
    * @return a <code>String</code> value
    */
   public String constraint_grammar ()
   {
      return "Custom Grammar";
   }
}
