/*
 *                           - COPYRIGHT NOTICE -
 *
 * The information contained herein and which follows is proprietary
 * information and is the property of PrismTech Corporation.  This
 * information is not to be divulged, released, copied, reproduced, or
 * conveyed to unauthorized personnel, companies, or other institutions
 * without the direct and expressed approval in writing of PrismTech
 * Corporation.
 *
 */

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosNotification.examples.News;

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;

/**
 * Utility class that can be used to obtain an event channel
 */
public class ChannelUtil
{
   private static EventChannel channel = null;

   /* The resolveName of the NotificationService */
   private static String resolveName = "NotificationService";

   /**
    * Obtains an event channel. Will get the event channel whose ID is 0 if
    * there is one, otherwise it will create a new one.
    * @param orb the ORB to use
    * @return the event channel
    */
   public static EventChannel getChannel (org.omg.CORBA.ORB orb)
      throws org.omg.CORBA.ORBPackage.InvalidName
   {
      if (ChannelUtil.channel == null)
      {
         EventChannelFactory factory;

         /* In order to avoid problems with VisiBroker when registering the
          name NotificationService, rename it */

         if ((com.prismt.openfusion.Version.getORB().toUpperCase().startsWith("VISIBROKER-52")) ||
            (com.prismt.openfusion.Version.getORB().toUpperCase().startsWith("VISIBROKER-60")))
         {
            resolveName = "OFNotificationService";
         }
         /* End of specific code to deal with VisiBroker and registering
          NotificationService name problem*/

         /* resolve the Notification Service */
         System.out.println
            ("Resolving initial references to "+ resolveName);
          org.omg.CORBA.Object notRef =
             orb.resolve_initial_references (resolveName);
          System.out.println ("Success");

         factory = EventChannelFactoryHelper.narrow (notRef);

         // Try and get the event channel whose ID is 0
         try
         {
            ChannelUtil.channel = factory.get_event_channel (0);
         }
         catch(ChannelNotFound cnf)
         {
            // No such event channel, so create a new one
            Property[] adm = new Property[0];
            Property[] qos = new Property[2];
            org.omg.CORBA.IntHolder id = new org.omg.CORBA.IntHolder ();

            qos[0] = new Property ();
            qos[0].name = ConnectionReliability.value;
            qos[0].value = orb.create_any ();
            qos[0].value.insert_short (Persistent.value);

            qos[1] = new Property ();
            qos[1].name = EventReliability.value;
            qos[1].value = orb.create_any ();
            qos[1].value.insert_short (Persistent.value);

            try
            {
               ChannelUtil.channel = factory.create_channel (qos,adm, id);
            }
            catch(Exception e)
            {
               System.out.println ("Error: " + e);
               return null;
            }
         }
      }
      return ChannelUtil.channel;
   }
}
