/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosNotification.examples.News;

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;

import com.prismt.orb.ObjectAdapter;

import java.util.Random;
import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * Creates structured events representing stock quotes and pushes them to
 * an event channel.
 */
public class StockQuote implements BureauOperations
{
   /**
    * All the companies about whom stock quotes will be generated
    */
   private String[] companies =
   {
      "apple",
      "boeing",
      "coca-cola",
      "disney",
      "eds",
      "ford",
      "geico",
      "honeywell",
      "ibm",
      "j. p. morgan",
      "kellogs",
      "levi strauss",
      "microsoft",
      "nike",
      "oracle",
      "phillips",
      "quantum",
      "reebok",
      "sun microsystems",
      "texas instruments",
      "unisys",
      "viacom",
      "wells fargo",
      "xerox",
      "york international"
   };

   /**
    * A map of companies and their most recent quote
    */
   private java.util.HashMap history = new java.util.HashMap ();

   private boolean stopped = false;
   private org.omg.CORBA.ORB orb;
   StructuredProxyPushConsumer proxy;

   /**
    * Create a new instance
    * @param orb the ORB to use
    */
   public StockQuote (org.omg.CORBA.ORB orb)
   {
      this.orb = orb;
      org.omg.CORBA.Object notRef = null;
      EventChannel channel = null;

      System.out.print ("Readying orb...");
      ObjectAdapter.ready (false);
      System.out.println ("Success");

      // Get the event channel
      try
      {
         System.out.print ("Getting channel...");
         channel = ChannelUtil.getChannel (this.orb);
         System.out.println ("Success");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName in)
      {
         System.out.println ("Could not resolve notification service!");
         System.exit (1);
      }

      if (channel == null)
      {
         System.out.println ("Could not get event channel 0");
      }

      // Get a structured proxy push consumer
      try
      {
         System.out.println ("Getting proxy...");
         SupplierAdmin sa = channel.default_supplier_admin ();
         ProxyConsumer pc = sa.obtain_notification_push_consumer
            (ClientType.STRUCTURED_EVENT, new org.omg.CORBA.IntHolder ());

         this.proxy = StructuredProxyPushConsumerHelper.narrow (pc);
         System.out.println ("Success");
      }
      catch(AdminLimitExceeded ale)
      {
         ale.printStackTrace ();
      }

      // Create a CORBA object for this instance
      System.out.print ("Creating supplier...");
      org.omg.CORBA.Object obj = ObjectAdapter.createTransient (this);
      StructuredPushSupplier supplier =
         StructuredPushSupplierHelper.narrow (obj);

      System.out.println ("Success");

      // Connect to the proxy structured push consumer
      try
      {
         System.out.print ("Connecting supplier to proxy...");
         this.proxy.connect_structured_push_supplier (supplier);
         System.out.println ("Success");
      }
      catch(org.omg.CosEventChannelAdmin.AlreadyConnected ac)
      {
         ac.printStackTrace ();
      }
   }

   /**
    * Publish structured events representing stock quotes
    */
   public void broadcast ()
   {
      while (!stopped)
      {
         try
         {
            StructuredEvent event = createNextEvent ();
            proxy.push_structured_event (event);
            System.out.println
               (
                "Sent event: " + event.remainder_of_body.extract_string () +
                " Priority:" +
                event.header.variable_header[1].value.extract_short ()
               );

            int random = Math.abs ((new Random ()).nextInt () % 2000);
            Thread.sleep (random);
         }
         catch (org.omg.CosEventComm.Disconnected d)
         {
            d.printStackTrace ();
         }
         catch(InterruptedException ie)
         {
            ie.printStackTrace ();
         }
      }
   }

   public void stop ()
   {
      stopped = true;
   }

   public void subscription_change (EventType[] added, EventType[] removed)
   {
   }

   public void disconnect_structured_push_supplier ()
   {
      System.out.println ("Disconnected from channel!");
      System.exit (1);
   }

   /**
    * Create a new structured event representing a randomly generated stock
    * quote.
    * @return the structured event
    */
   private StructuredEvent createNextEvent ()
   {
      StructuredEvent event = new StructuredEvent ();

      // Mark the event as being a stock quote
      EventType type = new EventType ("News","StockQuote");
      FixedEventHeader feh = new FixedEventHeader (type, "");

      Property[] variable = new Property[2];

      variable[0] = new Property ();
      variable[0].name = Timeout.value;
      variable[0].value = this.orb.create_any ();
      variable[0].value.insert_ulonglong ((long) 10 * 100 * 1000 * 60 * 5);

      variable[1] = new Property ();
      variable[1].name = Priority.value;
      variable[1].value = this.orb.create_any ();
      short prio = (short) Math.abs ((new Random ()).nextInt () % 10);
      variable[1].value.insert_short (prio);

      int random = Math.abs ((new Random ()).nextInt () % companies.length);
      String company = companies[random];

      Property[] filterable  = new Property[3];

      filterable[0] = new Property ();
      filterable[0].name = "bureau";
      filterable[0].value = this.orb.create_any ();
      filterable[0].value.insert_string ("Wall Street");

      filterable[1] = new Property ();
      filterable[1].name = "company";
      filterable[1].value = this.orb.create_any ();
      filterable[1].value.insert_string (company);

      filterable[2] = new Property ();
      filterable[2].name = "Priority";
      filterable[2].value = this.orb.create_any ();
      filterable[2].value.insert_short (prio);

      // Create the new stock quote
      String news = generateNextMessage (company);

      NamedPropertyRangeSeqHolder available =
         new NamedPropertyRangeSeqHolder ();

      // check the QoS properties for validity
      try
      {
         this.proxy.validate_event_qos (variable, available);
      }
      catch(UnsupportedQoS uqos)
      {
         System.out.println ("Exception: " + uqos);
         System.out.println
            (
             "Please set a correct value for the QoS and run the service " +
             "again!"
            );

         System.exit (1);
      }

      event.header = new EventHeader (feh,variable);
      event.filterable_data = filterable;
      event.remainder_of_body = this.orb.create_any ();

      // Add the stock quote to the structured event
      event.remainder_of_body.insert_string (news);

      return event;
   }

   /**
    * Randomly generate a new stock quote for a given company
    * @param company the company the new stock quote should refer to
    * @return the new stock quote
    */
   private String generateNextMessage (String company)
   {
      // Get the previous value for this company if available
      Short lastValue = (Short) history.get (company);

      if (lastValue == null)
      {
         lastValue = new Short ((short) 100);
      }

      // Randomly generate a new value
      short next = (short) Math.abs ((new Random ()).nextInt () % 500);

      // Store this new value for future reference
      history.put (company, new Short (next));

      String change;
      if (next < lastValue.shortValue ())
      {
         change = "dropped from";
      }
      else
      {
         change = "increased from";
      }

      // return an appropriate string
      return company + " " + change + " " + lastValue + " to " + next;
   }

   public static void main (String[] args) throws Exception
   {
      System.out.print ("Initialising the orb...");

      org.omg.CORBA.ORB orb = ObjectAdapter.init (args);
      System.out.println ("Success");
      StockQuote sq = new StockQuote (orb);
      System.out.println
         ("Ready to broadcast events. Press enter to start...");

      BufferedReader br =
         new BufferedReader (new InputStreamReader (System.in));
      br.readLine ();
      sq.broadcast ();
   }
}
