package com.prismt.cos.CosTime.examples.Event;

import com.prismt.orb.*;
import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;
import org.omg.CosTimerEvent.*;
import org.omg.CosTime.*;
import org.omg.TimeBase.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import com.prismt.cos.CosTime.TimeUtil;

public class TimerEventConsumer implements PushConsumerOperations
{
   private static org.omg.CORBA.ORB orb = null;
   private static org.omg.CORBA.Object ref = null;

   private NamingContext name = null;
   private EventChannelFactory factory = null;
   private TimerEventService time = null;

   ProxyPushSupplier supplier = null ;

   /**
    * Main method which creates a new TimerEventConsumer object,
    * then initialises it and waits for it to receive events
    */
   public static void main (String[] args)
   {
      TimerEventConsumer pc = new TimerEventConsumer ();
      pc.init (args);
      ObjectAdapter.ready( false ) ;
      pc.connect ();
      System.out.println ("\n--Waiting to receive events--\n");
      //ObjectAdapter.ready ();
      System.out.println( "Hit return to terminate" ) ;
      try
      {
         int userInput = System.in.read() ;
      }
      catch( java.io.IOException e )
      {
      }
      pc.cleanup() ;
      ObjectAdapter.shutdown() ;
   }

   /**
    * Initialises references to the ORB, the Notification Service,
    * the Name Service and the TimerEvent Service.
    *
    * @param argv arguments to pass to the ORB initialisation
    */
   public void init (String [] argv)
   {
      System.out.println ("\n--Initialising example--");

      //In order to make examples easier to run, for Orbacus-40 set a POAName
      if (com.prismt.openfusion.Version.getORB().
            toUpperCase().startsWith("ORBACUS-4"))
      {
         ObjectAdapter.setPOAName("OpenFusion.TimerEventConsumer");
      }

      orb = com.prismt.orb.ObjectAdapter.init (argv);
      ref = ObjectAdapter.createTransient (this);

      factory = getNotificationServer ();
      name = getNameServer ();
      time = getTimerEventServer ();

      writeIOR();
      System.out.println ("--Initialisation complete--");
   }

   /**
    * Obtains a proxy supplier from the notification service and
    * connects this push consumer to it.
    */
   public void connect ()
   {
      EventChannel channel = getEventChannel (factory);
      //      ProxyPushSupplier supplier = getSupplier (channel);
      supplier = getSupplier (channel);
      System.out.println ("--Connecting...--");

      try
      {
         supplier.connect_any_push_consumer (PushConsumerHelper.narrow (ref));
         System.out.println ("--Connected--");
      }
      catch (org.omg.CosEventChannelAdmin.AlreadyConnected ex)
      {
         System.err.println("Proxy is already connected, Exiting");
         System.exit(1);
      }
      catch (org.omg.CosEventChannelAdmin.TypeError ex)
      {
         /* not raised by the OpenFusion notification service */
      }
   }


   /**
    * Disconnects from the Notification Service, and removes this object's
    * reference from the Name Service.
    */
   public void cleanup()
   {
      supplier.disconnect_push_supplier() ;

      NameComponent[] key = new NameComponent[1];
      key[0] = new NameComponent();
      key[0].id = "TimerEventConsumer";
      key[0].kind = "Push Consumer";

      try
      {
         name.unbind (key);
      }
      catch (Exception exc)
      {
         System.out.println
         (
            "Problem unbinding consumer to the name service. Exiting."
         );
         System.exit (1);
      }
   }

   
   /**
    * This is where events are received by this push consumer.
    * The event data is printed out.
    */
   public void push (org.omg.CORBA.Any data)
   {
      TimerEventT value = TimerEventTHelper.extract (data);
      UTO uto = time.event_time (value);
      org.omg.CORBA.Any event = value.event_data;
      String s = event.extract_string ();

      System.out.print   ("(Received event) ");
      System.out.println ("data: " + s);
      System.out.println ("                 time: " + TimeUtil.UTOtoDate(uto));

   }

   /**
    * This method is required for all PushConsumers; it is not
    * implemented as it is not used in this example.
    */
   public void disconnect_push_consumer()
   {
      /* not implemented */
   }

   /**
    * This method is required for all PushConsumers; it is not
    * implemented as it is not used in this example.
    */
   public void connect_push_consumer()
   {
      /* not implemented */
   }

   /**
    * This method is required for all PushConsumers; it is not
    * implemented as it is not used in this example.
    */
   public void offer_change (EventType[] added, EventType[] removed)
   {
      /* not implemented */
   }

   /**
    * Given an EventChannelFactory this method obtains a reference
    * to an EventChannel, creating a new channel if necessary.
    *
    * @param factory notification service reference
    * @return an event channel
    */
   public EventChannel getEventChannel (EventChannelFactory factory)
   {
      EventChannel channel = null;

      try
      {
         channel= factory.get_event_channel (0);
      }
      catch (ChannelNotFound ex)
      {
         try
         {
            Property [] props1 = new Property [0];
            Property [] props2 = new Property [0];
            org.omg.CORBA.IntHolder id = new org.omg.CORBA.IntHolder ();
            channel = factory.create_channel (props1, props2, id);
         }
         catch (UnsupportedQoS exc)
         {
            /* can't be thrown, as we haven't specified any QoS */
         }
         catch (UnsupportedAdmin exc)
         {
            /* can't be thrown, as we're using the default admin */
         }

      }

      return channel;
   }

   /**
    * Given an EventChannel this method creates a new
    * ProxyPushSupplier on this channel.
    *
    * @param channel an event channel
    * @return the newly created proxy supplier
    */
   public ProxyPushSupplier getSupplier (EventChannel channel)
   {
      ProxySupplier supp = null;
      ProxyPushSupplier proxy = null;
      org.omg.CORBA.IntHolder id = new org.omg.CORBA.IntHolder ();
      ConsumerAdmin admin = channel.default_consumer_admin();
      try
      {
         supp = admin.obtain_notification_push_supplier (ClientType.ANY_EVENT, id);
      }
      catch (AdminLimitExceeded ex)
      {
         /* default admin doesn't have a limit */
      }
      proxy = ProxyPushSupplierHelper.narrow (supp);

      return proxy;
   }

   /**
    * Writes a reference to this consumer object to the name
    * service.  Other objects can then find this object using
    * the name service.  If the reference is already bound in
    * the name service then we overwrite it.
    */
   public void writeIOR ()
   {
      NameComponent[] key = new NameComponent[1];
      key[0] = new NameComponent();
      key[0].id = "TimerEventConsumer";
      key[0].kind = "Push Consumer";

      try
      {
         name.bind (key, ref);
      }
      catch (AlreadyBound ex)
      {
         try
         {
            name.unbind (key);
            name.bind (key, ref);
         }
         catch (Exception exc)
         {
            System.out.println (
               "Problem binding consumer to the name service. Exiting.");
            System.exit (1);
         }
      }
      catch (Exception ex)
      {
         System.out.println (
            "Problem binding consumer to the name service. Exiting.");
         System.exit (1);
      }
   }

   /*
    * Utility method to resolve the Notification Service
    *
    * @return reference to the notification server
    */
   public static EventChannelFactory getNotificationServer ()
   {
      org.omg.CORBA.Object obj = null;
      EventChannelFactory factory = null;

      try
      {
         obj = orb.resolve_initial_references("NotificationService");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println("Can't resolve Notification Service, Exiting");
         System.exit(1);
      }

      factory = EventChannelFactoryHelper.narrow(obj);

      return factory;
   }

   /*
    * Utility method to resolve the Name Service
    *
    * @return reference to the name server
    */
   public static NamingContext getNameServer ()
   {
      org.omg.CORBA.Object obj = null;
      NamingContext name = null;

      try
      {
         obj = orb.resolve_initial_references("NameService");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println("Can't resolve Name Service, Exiting");
         System.exit(1);
      }

      name = NamingContextHelper.narrow(obj);
      return name;
   }

   /*
    * Utility method to resolve the Timer Event Service
    *
    * @return reference to the timer event server
    */
   public static TimerEventService getTimerEventServer ()
   {
      org.omg.CORBA.Object obj = null;
      TimerEventService time = null;

      try
      {
         obj = orb.resolve_initial_references("TimerEventService");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println("Can't resolve Timer Event Service, Exiting");
         System.exit(1);
      }

      time = TimerEventServiceHelper.narrow(obj);
      return time;
   }
}
