/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

package com.prismt.cos.CosTrading.examples.Video;

import org.omg.CosTrading.*;
import org.omg.CosTradingRepos.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

import com.prismt.orb.*;

/**
 * Utility methods to obtain trading service objects.
 */

public class TradingUtil
{
   private static Lookup lookup = null;

   /**
    * Return the Trader's Lookup component
    * @return The Trader's Lookup component
    */
   public static Lookup getLookup ()
   {
      org.omg.CORBA.Object object = null;
      Lookup lookup = null;

      try
      {
         org.omg.CORBA.ORB orb = ObjectAdapter.init (new String[0]);
         object = orb.resolve_initial_references ("TradingService");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println ("Can't resolve Trading Service" + ex);
         System.exit (1);
      }
      catch (Exception ex)
      {
         System.err.println ("Can't resolve Trading Service" + ex);
         System.exit (1);
      }
      lookup = LookupHelper.narrow (object);

      return lookup;
   }

   /**
    * Return the Trader's ServiceTypeRepository component
    * @return The Trader's ServiceTypeRepository component
    */
   public static ServiceTypeRepository getTypeRepos ()
   {
      if (lookup == null)
      {
         lookup = getLookup ();
      }

      org.omg.CORBA.Object rep =  lookup.type_repos ();
      ServiceTypeRepository repository =
         ServiceTypeRepositoryHelper.narrow (rep);

      return repository;
   }

   /**
    * Return the Trader's Register component
    * @return The Trader's Register component
    */
   public static Register getRegister ()
   {
      if (lookup == null)
      {
         lookup = getLookup ();
      }

      Register register = lookup.register_if ();

      return register;
   }

   /**
    * Return the Trader's Admin component
    * @return The Trader's Admin component
    */
   public static Admin getAdmin ()
   {
      if (lookup == null)
      {
         lookup = getLookup ();
      }

      Admin admin = lookup.admin_if ();

      return admin;
   }

   /**
    * Return the Trader's Proxy component
    * @return The Trader's Proxy component
    */
   public static Proxy getProxy ()
   {
      if (lookup == null)
      {
         lookup = getLookup ();
      }

      Proxy proxy = lookup.proxy_if ();

      return proxy;
   }
}






