package org.jacorb.test.imrutility.imr;

import junit.framework.*;

import org.jacorb.imrutility.imr.*;

import java.io.*;

public class AutoActivationOnLookupTest  extends PersistentServerTestCase
{
    private String serverIOR;
    private final String ior = "server.ior";
    private final String home = System.getProperty("user.home");
    private final String separator = System.getProperty("file.separator");
    private Process proc;
    private RepositoryAdmin rep;
    private Activator act;
        
    public AutoActivationOnLookupTest(String name, PersistentServerSetup setup)
    {
        super(name, setup);
    }

    public static Test suite()
    {
        TestSuite suite = new TestSuite ("Auto Activation OnLookup Test");
                       
        PersistentServerSetup setup =
            new PersistentServerSetup
            ( 
                suite,
                "org.jacorb.test.imrutility.imr.Server"
            );
 
        suite.addTest (new AutoActivationOnLookupTest 
           ("testAutoActivationOnLookup", setup));
        
        // Return the setup, not the suite!
        return setup;
    }
 
    protected void tearDown()
    {
        System.err.println("********** tearDown for testAutoActivationOnLookup");
        try
        {
            if (proc != null)
            {
                proc.destroy();
            }
         
            if (rep != null)
            {       
                rep.shutdown(false);
            }
        
            TestUtility.removeIMRFiles();
            TestUtility.removeFile(serverIOR);
            TestUtility.destroyImR();
            TestUtility.removeActivatorFiles();
            TestUtility.stopActivator();
            Thread.sleep(1000);
        }
        catch (Exception ex)
        {
            /* don't do anything */
        }                            
    }
       
    public void testAutoActivationOnLookup()
    {
        String [] args = null;
        String [] actArgs = null;
        StringBuffer command = null;
        BufferedReader br = null;
        TestServer test = null;
        ServerName sname = null;
        ImplInfo [] info= null;
        CommandInfo cinfo = null;
                                                
        try
        {        
            /* Delete all files so no old data hanging around*/
            TestUtility.removeIMRFiles();
            
            /* Wait for files to be removed*/
            Thread.sleep(1000);
        }
        catch (Exception ex)
        {
            /* Don't do anything as this is not a problem*/
        }

        //Start the IMR        
        try
        {                     
            /* Set fixed port and also aliveness policy of ON_LOOKUP */
            args = new String[4];
            args[0] = "-p";
            args[1] = "23451";            
            args[2] = "-a";
            args[3] = "1";
                        
            rep = TestUtility.startImR(args, null);
            
            assertTrue("Failed to get ImR", rep != null);
        }
        catch(Exception ex)
        {
            fail("Exception getting ImR" + ex.getMessage());
        }

        /* get command to start server, start then stop server - gets IOR file*/        
        try
        {
            command = setup.getStartCommand();
            command.append(" -i ");
            serverIOR = home + separator + ior;
            command.append(serverIOR);
                        
            proc = TestUtility.startServer(command);
            
            proc.destroy();
        }
        catch (IOException ioe)
        {
            fail("Got IOException starting server " + ioe.getMessage());
        }
        catch (Exception ex)
        {
            fail("Got Exception starting server " + ex.getMessage());
        }        
        
        try
        {        
            br = new BufferedReader(new FileReader(serverIOR));

            // get object reference from command-line argument file
            org.omg.CORBA.Object obj =
                TestUtility.getImRORB().string_to_object(br.readLine());

            br.close();

            //narrow to correct interface
            test = TestServerHelper.narrow(obj);
            
            test.ping();
                        
            fail("Expected an exception pinging the server - no activator");
        }
        catch (Exception ex)
        {
            // do nothing - Exception expected cannot 
            // start server as no Activator registered 
        }

        try
        {
            sname = new ServerName();
            sname.impl("test_imr");
        
            actArgs = new String [0];     
            rep.removeImpl(sname, null);

            act = TestUtility.startActivator(actArgs, null);
            
            assertTrue("Activator not found", act != null);        
            
            rep.addImpl
                (new ImplInfo
                    (sname, 
                     false, 
                     LoadBalancingPolicy.from_int(0), 
                     ActivationMode.from_int(2)
                     ), 
                     act);  
                    
            act.addCommand
                (new CommandInfo(sname, command.toString()));

            test.ping();
            
            fail("Expected an exception pinging the server - invalid activation mode");                
        }
        catch (Exception ex)
        {
            // do nothing the activation mode does not allow auto-activation
        }
        
        try
        {
            rep.removeImpl(sname, null);
            
            rep.addImpl
                (new ImplInfo
                    (sname, 
                     false, 
                     LoadBalancingPolicy.from_int(0), 
                     ActivationMode.from_int(3)
                     ), 
                     act);  
                            
            act.removeCommand(sname);
            
            act.addCommand
                (new CommandInfo(sname, command.toString()));                
                
            test.ping();
        }
        catch (Exception ex)
        {
            fail ("Unexpected exception - Server should have started");    
        }        
        
        test.shutdown();
    }
}
