
package org.jacorb.imrutility.imr.persistence;


public class DataLock 
{
    /** Constant to signify Shared data lock */
    public static final int SHARED = 0;
    
    /** Constant to signify Exclusive data lock */
    public static final int EXCLUSIVE = 1;
    
    /** A count of the number of shared locks held*/
    private int sharedCount;
    
    /** A count of the number of exclusive locks held*/
    private int exclusiveCount;
    
    /** Indicates that there are requests for locks waiting */
    private boolean waiting = false;

    /**
     * The constructor.
     */
    public DataLock() 
    {
        sharedCount = 0;
        exclusiveCount = 0;
    }

    /**
     * This method tries to aquire a shared lock. It blocks
     * until the exclusive lock is released.
     */
    public synchronized void getLock(int lockType)
    {
        if (lockType == SHARED)
        {
            while (exclusiveCount > 0 && waiting)
            {  
                try
                {
                    wait();
                }
                catch (java.lang.Exception _e)
                {  

                }
            }
        
            sharedCount++;
        }
        else
        {
            while (sharedCount > 0 || exclusiveCount > 0)
            {
                try
                {
                    waiting = true;
                    wait();
                }
                catch (InterruptedException iex)
                {

                }
            }
            
            exclusiveCount++;
            waiting = false;        
        }
    }

    /**
     * Release a lock. Unblocks threads waiting for
     * access.
     */
    public synchronized void releaseLock(int lockType)
    {
        if (lockType == SHARED)
        {
           if (--sharedCount == 0)
           {
               notifyAll();
           }
        }
        else if (--exclusiveCount == 0)
        {
            notifyAll();
        }
    }
}
