package org.jacorb.imrutility.imr.util;


import org.jacorb.imrutility.imr.AlreadyRegistered;
import org.jacorb.imrutility.imr.CommandInfo;
import org.jacorb.imrutility.imr.ActivationMode;
import org.jacorb.imrutility.imr.Activator;
import org.jacorb.imrutility.imr.ActivatorHelper;
import org.jacorb.imrutility.imr.NotFound;
import org.jacorb.imrutility.imr.RepositoryAdmin;
import org.jacorb.imrutility.imr.StartupFailed;
import org.jacorb.imrutility.imr.UnknownServer;
import org.jacorb.imrutility.imr.ServerName;

import java.util.StringTokenizer;
import java.util.Vector;

import org.omg.CORBA.ORB;

/**
 * @author cj
 *
 * This class is used to manage the <code>Activator</code> that
 * is responsible for starting implementations dynamically.
 */
public class ActivatorManager
{
    /** The ORB */
    private static ORB orb;

    /** The Activator object */
    private static Activator activator = null;
   
    /**
     * Main method
     */
    public static void main (String args[])
    {
        orb = org.omg.CORBA.ORB.init (args, null);

        if (args.length > 0)
        {
            if (args[0].equals ("-h") || args[0].equals("--help"))
            {
                usage();
            }
            else
            {
                getActivator();
                
                for (int i = 0; i < args.length; i++)
                {                                                 
                    if (args[i].equals ("-l") || args[i].equals ("--list"))
                    {
                        list (args);
                        break;
                    }
                    else if (args[i].equals ("-a") || args[i].equals ("--add"))
                    {
                        addCommand (args);
                        break;
                    }
                    else if (args[i].equals ("-r") || args[i].equals ("--remove"))
                    {
                        removeCommand (args);
                        break;
                    }
                    else if (args[i].equals ("-s") || args[i].equals ("--start"))
                    {
                        startServer (args);
                        break;
                    }
                }               
            }
        }
        else
        {
            usage();
        }
    }

    /**
     * Lists the commands registered with the Activator
     */
    private static void list(String [] args)
    {
        String name;
       
        if (args.length <= 2)
        {
            listAll();                        
        }       
   
        /* Code temporarily removed as the IDL does not support 
         * supplying a name
         */
              
       /*
        else
        {
            if (args[i].equals("-n") || args[i].equals("--name"))
            {
                name = args[2];
                listSelected(name);
            }
            else if (args[i].equals("-p") || args[i].equals("--poa"))
            {
                System.out.println("Listing of POA start commands is not yet implemented");
                System.exit(0);
            }       
        }
        */
    }
   
    /**
     * Lists the commands for a specific Implementation name - Note this method
     * will not be invoked at present as there is not operation in the IDL to
     * handle the request and it was to late in the day to start changing the
     * IDL - will be changed at next release
     */
    private static void listSelected(String name)
    {
        String command;
        CommandInfo [] infoArray = null;
        CommandInfo cinfo = null;
        ServerName sname = null;
      
        try
        {
            sname = new ServerName();
            sname.impl(name);
         
            infoArray = activator.listPOACommands(sname);

            System.out.println( "Name            : " + name + "\n");
         
            if (infoArray.length > 0)
            {
                for (int i = 0; i < infoArray.length; i++)
                {
                    cinfo = infoArray [i];
               
                    System.out.println( "Command         : " + cinfo.cmd + "\n");
                }
            }
            else
            {
                System.out.println ("No commands registered");
            }
        }
        catch (Exception ex)
        {
            System.out.println ("Exception listing commands\n");
            ex.printStackTrace();
        }
    }
   
    /**
     * Lists the commands registered with the Activator
     */
    private static void listAll()
    {
        String command;
        String name;
        CommandInfo [] infoArray = null;
        CommandInfo cinfo = null;
        ServerName sname = null;
      
        try
        {
            infoArray = activator.listCommands();

            if (infoArray.length > 0)
            {
                for (int i = 0; i < infoArray.length; i++)
                {
                    cinfo = infoArray [i];

                    name = getName(cinfo.name);
               
                    System.out.println( "Name            : " + name + "\n");
                    System.out.println( "Command         : " + cinfo.cmd + "\n");
                }
            }
            else
            {
                System.out.println ("No commands registered");
            }
        }
        catch (Exception ex)
        {
            System.out.println ("Exception listing commands\n");
            ex.printStackTrace();
        }
    }

    /**
     * Add a command.
     * @param args contains the implementation name and the command to
     * start that implementation
     */
    private static void addCommand (String [] args)
    {
        RepositoryAdmin rep = null;
        String command;
        String name = null;
        CommandInfo cinfo = null;
        StringBuffer sb = new StringBuffer();
        String cmd = "";
        ServerName sname;
        String mode = null;
      
        if (args.length >= 5)
        {
            for (int i = 1; i < args.length; i++)
            {
                if (i < args.length - 1 || (cmd.equals("c") && i == args.length - 1))
                {                    
                    if (args[i].equals("-n") || args[i].equals("--name"))
                    {
                        cmd = "n";
                        name = args[++i];
                    }
                    
                    if (args[i].equals("-p") || args[i].equals("--poa"))
                    {
                        System.out.println("Addition of POAs is not yet implemented");
                        System.exit(0);
                    }
                    else if (args[i].equals("-c") || args[i].equals("--command"))
                    {
                        cmd = "c";
                        sb.append(args[++i]);
                        sb.append(' ');
                    }
                    else
                    {
                        if(cmd.equals("c"))
                        {
                            sb.append(args[i]);

                            if (i < (args.length - 1))
                            {
                                sb.append( ' ' );
                            }
                        }
                    }
                }
            }                                
                        
            command = sb.toString();
                
            if (command.length() == 0)
            {
                System.out.println("You must provide the start up command");
                System.exit(0);
            }    
            
            sname = new ServerName();
            sname.impl(name);
          
            /* If mode not provided set to default of NORMAL*/
            if (mode == null)
            {  
                mode = "0";
            }
                    
            cinfo = new CommandInfo (sname, command);
        }            
        else
        {
            usage();
        }

        try
        {
            activator.addCommand (cinfo);
            System.out.println ("Command added");
        }
        catch (AlreadyRegistered ar)
        {
            System.out.println ("Command already registered");
        }
        catch (Exception ex)
        {
            System.out.println ("Exception adding command \n");
            ex.printStackTrace();
        }
    }

    /**
     * Removes a command
     */
    private static void removeCommand (String [] args)
    {
        String name = null;
        ServerName sname;
      
        if (args.length == 0)
        {
            usage();
        }
        else
        {
            for (int i = 0;  i < args.length; i++)
            {
                if (args[i].equals("-n") || args[i].equals("--name"))
                {
                    if (i < args.length - 1)
                    {
                        name = args[++i];
                    }
                }
                else if (args[i].equals("-p") || args[i].equals("--poa"))
                {
                    System.out.println("Removal of POAs is not yet implemented");
                    System.exit(0);
                }
            }
        }

        if (name == null)
        {
            System.out.println("You must provide the Server name");
            System.exit(0);
        }
                    
        try
        {
            sname = new ServerName();
            sname.impl(name);
         
            activator.removeCommand (sname);
            System.out.println ("Command " + name + " removed");
        }
        catch (NotFound nf)
        {
            System.out.println (name + " :: " + nf.getMessage());
        }
        catch (Exception ex)
        {
            System.out.println ("Exception removing command \n");
            ex.printStackTrace();
        }
    }

    /**
     * Method to instigate the starting of a Server
     */
    private static void startServer (String [] args)
    {
        String name = null;
        ServerName sname = null;

        for (int i = 0; i < args.length; i++)
        {      
            if (args[i].equals("-n") || args[i].equals("--name"))
            {
                if (i < args.length - 1)
                {
                    name = args[++i];
                }
            }
            else if (args[1].equals("-p") || args[1].equals("--poa"))
            {
                System.out.println("Starting of POA is not yet implemented");
                System.exit(0);
            }   
        }

        if (name == null)
        {
            System.out.println("You must provide the Server name");
            System.exit(0);
        }
        
        try
        {
            sname = new ServerName();
            sname.impl(name);
         
            activator.startServer (sname, true);            
            System.out.println ("Server " + name + " started");
        }
        catch (UnknownServer us)
        {     
            System.out.println( us.name + " :: " + us.getMessage());
        }
        catch (StartupFailed sf)
        {
            System.out.println( sf.reason );
        }
        catch (Exception ex)
        {
            System.out.println ("Exception starting server \n");
            ex.printStackTrace();
        }
        finally
        {
            try
            {
                activator.serverStarted(sname);
            }
            catch (UnknownServer us)
            {}
            catch (Exception ex)
            {}
        }
    }

    /**
     * Method to get a string display name from a ServerName object
     */
    private static String getName(ServerName sname)
    {
        String name = null;
        String [] [] names = null;
        String [] pnames = null;
        
        if (sname.discriminator())
        {
            name = sname.impl();    
        }
        else
        {
            names = sname.poas();
                
            if (names != null)
            {        
                for (int i = 0; i < names.length; i++)
                {                
                    for (int j = 0; j < names[i].length; j++)
                    {
                        name += "/";
                        name += names[i][j];
                    }
                }
            }
        }
        
        return name;
    }  

    /**
     * Method to create a ServerName from an input string
     */    
    private static ServerName getServerName(String in_name)
    {
        StringTokenizer st = new StringTokenizer(in_name, "/");
        String name = null;
        String [] [] poas = null;
        String [] ipoas;
        Vector vpoas = new Vector();
        ServerName sname = new ServerName();
        
        if(in_name.startsWith("/"))
        {
            while (st.hasMoreTokens())
            {
                vpoas.add(st.nextToken());
            }

            ipoas = new String[vpoas.size()];

            for (int i = 0; i < vpoas.size(); i++)
            {
                ipoas [i] = (String) vpoas.elementAt(i);
            }
            
            poas = new String [1] [ipoas.length];
            poas[0] = ipoas;
        }
        else
        {
            name = st.nextToken();
        }
        
        if (name != null)
        {
            sname.impl(name);
        }
        else
        {
            sname.poas(poas);
        }
        
        return sname;
    } 
    
   /**
    * Help to show how to use the ActivatorManager
    */
   private static void usage()
    {
        System.out.println("Usage");
        System.out.println("--------------------------------------------------------------");
        System.out.println(" [ -l | --list ]        : lists all commands registered with  ");
        System.out.println("                          the activator                       ");
        System.out.println();        
        System.out.println("--------------------------------------------------------------");
        System.out.println(" [ -a | --add ]         : adds a command for a server using   ");
        System.out.println("                          the following options:-             ");
        System.out.println();
        System.out.println("   [ -n | --name ] <name> the name of the server              ");
        System.out.println();        
        System.out.println("   [ -p  | --poa ] <poa name> Not yet implemented             ");
        System.out.println();
        System.out.println("   [ -c | --command ] <command>                               ");
        System.out.println("                          the command to be used to start the ");
        System.out.println("                          the server                          "); 
        System.out.println("                                                              ");
        System.out.println("--------------------------------------------------------------");
        System.out.println(" [ -r | -- remove ]     : removes the command registered for  ");
        System.out.println("                          the given name/poa name uses the    ");
        System.out.println("                          following options                   ");
        System.out.println("   [-n, | --name] <name>                                      ");
        System.out.println();
        System.out.println("   [ -p  | --poa ] <poa name> Not yet implemented             ");        
        System.out.println("--------------------------------------------------------------");
        System.out.println(" [ -s | --start ]       : starts the server using the command ");
        System.out.println("                          previously registered for the given ");
        System.out.println("                          name/poa name, uses the following   ");
        System.out.println("                          options:                            ");
        System.out.println("   [-n, --name] <name>                                        ");
        System.out.println();
        System.out.println("   [ -p  | --poa ] <poa name> Not yet implemented             ");                
        System.out.println("--------------------------------------------------------------");
        System.out.println(" [ -h | --help ]          displays this help                  ");
        System.out.println("--------------------------------------------------------------");
        System.exit (0);
    }

    /**
     *  Gets a reference to the Activator
     */
    private static void getActivator()
    {
        try
        {
            org.omg.CORBA.Object obj = orb.resolve_initial_references ("IMRActivator");
          
            activator = ActivatorHelper.narrow (obj);
          
            if (activator == null)
            {
                System.out.println ("Unable to locate the Activator");
            }
        }
        catch(Exception ex)
        {
            System.out.println("Exception locating Activator");
            System.out.println(ex.getMessage());
        }
      
        if(activator == null)
        {
            System.exit(0);
        }
    }
}
