package org.jacorb.orb.giop;

/*
 *        JacORB - a free Java ORB
 *
 *   Copyright (C) 1997-2003  Gerald Brose.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// Imports
import java.util.List;
import org.jacorb.util.Environment;
import org.omg.CORBA.INTERNAL;

/**
 * @author Nicolas Noffke
 * @version $Id: ClientGIOPConnection.java,v 1.3 2004/05/19 16:22:32 rnc Exp $
 */

public class ClientGIOPConnection
    extends GIOPConnection
{
    private boolean ignore_pending_messages_on_timeout = false;

    public ClientGIOPConnection( org.omg.ETF.Profile profile,
                                 org.omg.ETF.Connection transport,
                                 RequestListener request_listener,
                                 ReplyListener reply_listener,
                                 StatisticsProvider statistics_provider )
    {
        super( profile, transport, request_listener, reply_listener, statistics_provider );

        //default to "off" is handled internally by Environment.isPropertyOn()
        ignore_pending_messages_on_timeout =
            Environment.isPropertyOn("jacorb.connection.client.timeout_ignores_pending_messages");
    }

    protected void readTimedOut()
    {
        synchronized( pendingUndecidedSync )
        {
            if (ignore_pending_messages_on_timeout)
            {
                this.streamClosed();
            }
            else if (! hasPendingMessages())
            {
                closeAllowReopen();
            }
        }
    }

    protected void streamClosed()
    {
        closeAllowReopen();

        if( connection_listener != null )
        {
            connection_listener.streamClosed();
        }
    }

    public void closeAllowReopen()
    {
        try
        {
            synchronized (connect_sync)
            {
                getWriteLock();
                transport.close();
                // We expect that the same transport can be reconnected
                // after a close, something that the ETF draft isn't
                // particularly clear about.
            }
        }
        finally
        {
            releaseWriteLock();
        }
    }


    /**
     * <code>addContextKey</code> caches the key, oid and scopes from ServerRequest
     * in order to speed communication between JacORB implementations.
     *
     * It is only used on the server side - see ServerGIOPConnection.
     *
     * @param object_key a <code>byte[]</code> value
     * @param oid a <code>byte[]</code> value
     * @param scopes a <code>List</code> value
     * @return a <code>byte[]</code> value
     */
    public byte[] addContextKey(byte []object_key, byte []oid, List scopes)
    {
        throw new INTERNAL ("Server ContextKey ops should not have been called");
    }

    /**
     * <code>getContextKey</code> will return a cached ContextKey value
     * for that mini-key if one exists.
     *
     * It is only used on the server side - see ServerGIOPConnection.
     *
     * @param key a <code>byte[]</code> value
     * @return a <code>ContextKey</code> value
     */
    public GIOPConnectionManager.ContextKey getContextKey(byte []key)
    {
        // Can't throw INTERNAL as above as this might be called from BiDir
        // clients. Return null for them.
        return null;
    }
}// ClientGIOPConnection
