package org.jacorb.test.imrutility.imr;

import junit.framework.*;

import org.jacorb.imrutility.imr.*;
import org.jacorb.util.Environment;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.FileReader;
import java.util.Properties;

public class ImRFailoverTest extends PersistentServerTestCase
{
    private String serverIOR;
    private final String ior = "server.ior";
    private final String ior2 = "server2.ior";
    private final String home = System.getProperty("user.home");
    private final String separator = System.getProperty("file.separator");
    private Process proc;
    private RepositoryAdmin rep;

    public ImRFailoverTest(String name, PersistentServerSetup setup)
    {
        super(name, setup);
    }

    public static Test suite()
    {
        TestSuite suite = new TestSuite ("ImR Failover Test");
                       
        PersistentServerSetup setup =
            new PersistentServerSetup
            ( 
                suite,
                "org.jacorb.test.imrutility.imr.Server"
            );
 
        suite.addTest (new ImRFailoverTest ("testImRFailover", setup));
        
        // Return the setup, not the suite!
        return setup;
    }
 
    protected void tearDown()
    {
        System.err.println("********** tearDown for testImRFailover");
        try
        {
            if (proc != null)
            {
                proc.destroy();
            }
         
            if (rep != null)
            {       
                rep.shutdown(false);
            }
        
            TestUtility.removeIMRFiles();
            TestUtility.removeFile(serverIOR);
            TestUtility.destroyImR();
            Thread.sleep(1000);
        }
        catch (Exception ex)
        {
            /* don't do anything */
        }                            
    }
       
    public void testImRFailover()
    {
        Process proc2 = null;
        ImplInfo [] infos = null;
        StringBuffer command = null;
        ServerInfo [] sinfos = null;
        POAInfo [] poas = null;
        TestServer test = null;
        ServerName sname = null;
        long originalID = 0;
        String [] args = null;
      
        try
        {        
            TestUtility.removeIMRFiles();
            
            /* Wait for files to be removed*/
            Thread.sleep(1000);
        }
        catch (Exception ex)
        {
            /* Don't do anything as this is not a problem*/
        }

        //Start the IMR        
        try
        {                     
            args = new String[2];
            args[0] = "-u";
            args[1] = "23456";            
            
            rep = TestUtility.startImR(args, null);
            
            assertTrue("Failed to get ImR", rep != null);
        }
        catch(Exception ex)
        {
            fail("Exception getting ImR" + ex.getMessage());
        }
        
        try
        {
            Environment.setProperty("jacorb.imr.heartbeat", "5000"); 
            Environment.setProperty("jacorb.imr.udp_port", "23456");                       

            command = setup.getStartCommand();
            command.append(" -i ");
            serverIOR = home + separator + ior;
            command.append(serverIOR);
                        
            proc = TestUtility.startServer(command);
                    
            sname = new ServerName();
            sname.impl("test_imr");
                
            sinfos = rep.listServers(sname);
                        
            assertTrue("Server not registered", sinfos.length == 1);
        
            originalID = sinfos[0].id;                     
        }
        catch (IOException ioe)
        {
            fail("Got IOException starting server " + ioe.getMessage());
        }
        catch (Exception ex)
        {
            fail("Got Exception starting server " + ex.getMessage());
        }
                
        //Stop and restart the IMR        
        try
        {                         
            TestUtility.destroyImR();

            TestUtility.removeFile
                (Environment.getProperty("jacorb.imr.ior_file"));                     
                    
            rep = TestUtility.startImR(args, null);
                        
            assertTrue("Failed to get ImR", rep != null);

            /* Give server time to re-register with the new IMR*/            
            Thread.sleep(12000);
            
            sinfos = rep.listServers(sname);
        
            assertTrue("Server not registered", sinfos.length == 1);
        
            assertTrue("IDs don't match", sinfos[0].id == originalID);                    
            
        }
        catch(Exception ex)
        {
            fail("Exception getting ImR" + ex.getMessage());
        } 
        
        try
        {
            command = setup.getStartCommand();
            command.append(" -i ");
            serverIOR = home + separator + ior2;
            command.append(serverIOR);
                        
            proc2 = TestUtility.startServer(command);
            sinfos = rep.listServers(sname);
            assertTrue("Second server got registered", sinfos.length == 1);                        
        }       
        catch (Exception ex)
        {
            //fail("Got an Exception starting the server " + ex.getMessage());
        }           
    }
}
