package org.jacorb.test.imrutility.imr;


import org.jacorb.imrutility.imr.*;

import junit.framework.*;

import java.io.IOException;
import java.util.Properties;

public class ServerRestartWithDataFile extends PersistentServerTestCase
{   
    private String serverIOR;
    private final String ior = "server.ior";
    private final String home = System.getProperty("user.home");
    private final String separator = System.getProperty("file.separator");
    private RepositoryAdmin rep;
    private Process proc;
    
    public ServerRestartWithDataFile(String name, PersistentServerSetup setup)
    {
        super(name, setup);
    }

    public static Test suite()
    {
        TestSuite suite = new TestSuite ("Server Restart With DataFile Test");
        
        Properties clientProps = new Properties();
               
        PersistentServerSetup setup =
            new PersistentServerSetup
            ( 
                suite,
                "org.jacorb.test.imrutility.imr.Server"
            );

        suite.addTest
            (new ServerRestartWithDataFile
                ("serverRestartWithDataFile", setup)); 
        
        // Return the setup, not the suite!
        return setup;
    }

    protected void tearDown()
    {
        System.err.println("********** tearDown for ServerRestartWithDataFile"); 
        try
        {
            if (proc != null)
            {
                proc.destroy();
            }
        
            if (rep != null)
            {    
                rep.shutdown(false);
            }
            
            TestUtility.removeIMRFiles();
            TestUtility.removeFile(serverIOR);
            TestUtility.destroyImR();
            Thread.sleep(1000);
        }
        catch (Exception ex)
        {
            /* don't do anything */
        }

    }
           
    /**
     * This tests starting of a server when details have been loaded from
     * the ImR data file.  Only implementation details should be loaded
     * as the original server is no longer running.  The server should
     * be successfully registered
     */    
    public void serverRestartWithDataFile()
    {
        ImplInfo [] infos = null;
        StringBuffer command = null;
        ServerInfo [] sinfos = null;
        POAInfo [] poas = null;
        TestServer test = null;
        String [] args = null;
                              
        /* do not remove data files as we want them to exist for this test*/
      
        /* Start the IMR         */
        try
        {             
            args = new String[0];
            rep = TestUtility.startImR(args, null);
                        
            assertTrue("Failed to get ImR", rep != null);
        }
        catch(Exception ex)
        {
            fail("Exception getting ImR" + ex.getMessage());
        }                

        infos = rep.listAllImpls();
                
        assertTrue
        (
            "Implementation details not loaded from persistence", 
            infos.length == 1
        );

        try
        {
            sinfos = rep.listServers(infos[0].name);
        }
        catch (NotFound nf)
        {
            fail("Server not found as expected");
        }
        
        assertTrue
        (
            "Servers registered erroneously",
            sinfos.length == 0
        );

        /* Restart the server */
        try
        {
            command = setup.getStartCommand();
            command.append(" -i ");
            serverIOR = home + separator + ior;
            command.append(serverIOR);
            
            proc = TestUtility.startServer(command);
        
            /* Wait for the IOR file to be written & Server registered*/
            Thread.sleep(2000);                        
        }
        catch (IOException ioe)
        {
            fail("Got IOException starting server " + ioe.getMessage());
        }
        catch (Exception ex)
        {
            fail(ex.getMessage());
        }

        try
        {
            ServerName sname = new ServerName();
            sname.impl("test_imr");
            
            sinfos = rep.listServers(sname);
                
            assertTrue
            (
                "Incorrect number of servers registered",
                sinfos != null && sinfos.length == 1
            );
        
            assertTrue
            (
                "ServerStatus incorrect",
                sinfos[0].state.value() == ServerState._ACTIVE
            );
        
            assertTrue
            (
                "Server is null",
                sinfos[0].srv != null
            );
        }
        catch (Exception ex)
        {
            fail("Exception listing servers");
        }                
    }            
}
